﻿using Benday.BuildUtilities.Core.ConsoleUi;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;

namespace Benday.BuildUtilities.Core.Tests
{
    [TestClass]
    public class JsonEditorUtilityFixture : UnitTestBase
    {
        [TestInitialize]
        public void OnTestInitialize()
        {
            _PathToSampleConfigFile = CreateSampleJsonConfigFile();
        }

        string _PathToSampleConfigFile;

        [TestMethod]
        public void GetValue_OneLevel()
        {
            var expected = "FirstLevelValue";

            var actual = JsonEditorUtility.GetValue(
                _PathToSampleConfigFile,
                "FirstLevel");

            Assert.AreEqual<string>(expected, actual, "Wrong value.");
        }

        [TestMethod]
        public void GetValue_TwoLevels()
        {
            var expected = "default-connection-string-value";

            var actual = JsonEditorUtility.GetValue(
                _PathToSampleConfigFile, 
                "ConnectionStrings", "DefaultConnectionString");

            Assert.AreEqual<string>(expected, actual, "Wrong value.");
        }

        [TestMethod]
        public void GetValue_ThreeLevels()
        {
            var expected = "Debug";

            var actual = JsonEditorUtility.GetValue(
                _PathToSampleConfigFile,
                "Logging", "LogLevel", "Default");

            Assert.AreEqual<string>(expected, actual, "Wrong value.");
        }

        [TestMethod]
        public void SetValue_OneLevel()
        {
            var expected = "new-FirstLevelValue";

            JsonEditorUtility.SetValue(
                _PathToSampleConfigFile,
                "FirstLevel", expected);

            var actual = JsonEditorUtility.GetValue(
                _PathToSampleConfigFile,
                "FirstLevel");

            Assert.AreEqual<string>(expected, actual, "Wrong value.");
        }

        
        [TestMethod]
        public void SetValue_TwoLevels()
        {
            var expected = "new-default-connection-string-value";

            JsonEditorUtility.SetValue(
                _PathToSampleConfigFile,
                "ConnectionStrings", "DefaultConnectionString", expected);

            var actual = JsonEditorUtility.GetValue(
                _PathToSampleConfigFile,
                "ConnectionStrings", "DefaultConnectionString");

            Assert.AreEqual<string>(expected, actual, "Wrong value.");
        }

        [TestMethod]
        public void SetValue_ThreeLevels()
        {
            var expected = "new-Debug";

            JsonEditorUtility.SetValue(
                _PathToSampleConfigFile,
                "Logging", "LogLevel", "Default", expected);

            var actual = JsonEditorUtility.GetValue(
                _PathToSampleConfigFile,
                "Logging", "LogLevel", "Default");

            Assert.AreEqual<string>(expected, actual, "Wrong value.");
        }
    }
}
